@@ -71,10 +71,10 @@ struct color_t
 {
   static color_t from_ansi (unsigned int x)
   {
-    color_t c = {(0xFF<<24) | ((0xFF*(x&1))<<16) | ((0xFF*((x >> 1)&1))<<8) | (0xFF*((x >> 2)&1))};
+    color_t c = {(0xFFu<<24) | ((0xFFu*(x&1))<<16) | ((0xFFu*((x >> 1)&1))<<8) | (0xFFu*((x >> 2)&1))};
     return c;
   }
-  unsigned int to_ansi (void)
+  unsigned int to_ansi ()
   {
     return ((v >> 23) & 1) | ((v >> 14)&2) | ((v >> 5)&4);
   }
@@ -110,7 +110,7 @@ struct image_t
 		own_data (true),
 		data ((color_t *) malloc (sizeof (data[0]) * width * height)),
 		stride (width) {}
-  ~image_t (void)
+  ~image_t ()
   { if (own_data) free (data); }
 
   color_t &operator () (unsigned int x, unsigned int y)
@@ -133,7 +133,7 @@ struct image_t
 	for (unsigned int col = 0; col < w; col++)
 	  *q++ = *p++;
       else {
-        unsigned int limit = width - x;
+	unsigned int limit = width - x;
 	for (unsigned int col = 0; col < limit; col++)
 	  *q++ = *p++;
 	p--;
@@ -161,7 +161,7 @@ struct biimage_t
 		height (height),
 		bg (0), fg (0), unicolor (true),
 		data ((uint8_t *) malloc (sizeof (data[0]) * width * height)) {}
-  ~biimage_t (void)
+  ~biimage_t ()
   { free (data); }
 
   void set (const image_t &image)
@@ -171,17 +171,17 @@ struct biimage_t
     int freq[8] = {0};
     for (unsigned int y = 0; y < height; y++)
       for (unsigned int x = 0; x < width; x++) {
-        color_t c = image (x, y);
-        freq[c.to_ansi ()]++;
+	color_t c = image (x, y);
+	freq[c.to_ansi ()]++;
       }
     bg = 0;
     for (unsigned int i = 1; i < 8; i++)
       if (freq[bg] < freq[i])
-        bg = i;
+	bg = i;
     fg = 0;
     for (unsigned int i = 1; i < 8; i++)
       if (i != bg && freq[fg] < freq[i])
-        fg = i;
+	fg = i;
     if (fg == bg || freq[fg] == 0) {
       fg = bg;
       unicolor = true;
@@ -202,7 +202,7 @@ struct biimage_t
     int dd = diff.dot (diff);
     for (unsigned int y = 0; y < height; y++)
       for (unsigned int x = 0; x < width; x++) {
-        int d = diff.dot (image (x, y).diff (bgc));
+	int d = diff.dot (image (x, y).diff (bgc));
 	(*this)(x, y) = d < 0 ? 0 : d > dd ? 255 : lround (d * 255. / dd);
       }
   }
@@ -223,7 +223,7 @@ struct biimage_t
   uint8_t * const data;
 };
 
-const char *
+static const char *
 block_best (const biimage_t &bi, bool *inverse)
 {
   assert (bi.width  <= CELL_W);
@@ -287,13 +287,13 @@ block_best (const biimage_t &bi, bool *inverse)
       unsigned int s;
       s = row_sum[i] + total_i - row_sum_i[i];
       if (s < best_s) {
-        best_s = s;
+	best_s = s;
 	best_i = i;
 	best_inv = false;
       }
       s = row_sum_i[i] + total - row_sum[i];
       if (s < best_s) {
-        best_s = s;
+	best_s = s;
 	best_i = i;
 	best_inv = true;
       }
@@ -319,13 +319,13 @@ block_best (const biimage_t &bi, bool *inverse)
       unsigned int s;
       s = col_sum[i] + total_i - col_sum_i[i];
       if (s < best_s) {
-        best_s = s;
+	best_s = s;
 	best_i = i;
 	best_inv = true;
       }
       s = col_sum_i[i] + total - col_sum[i];
       if (s < best_s) {
-        best_s = s;
+	best_s = s;
 	best_i = i;
 	best_inv = false;
       }
@@ -353,7 +353,7 @@ block_best (const biimage_t &bi, bool *inverse)
 	} else
 	  qs += quad[i][j];
     if (qs < score) {
-      const char *c = NULL;
+      const char *c = nullptr;
       bool inv = false;
       switch (q) {
 	case 1:  c = "▟"; inv = true;  break;
@@ -396,15 +396,15 @@ ansi_print_image_rgb24 (const uint32_t *data,
       image.copy_sub_image (cell, col * CELL_W, row * CELL_H, CELL_W, CELL_H);
       bi.set (cell);
       if (bi.unicolor) {
-        if (last_bg != bi.bg) {
+	if (last_bg != bi.bg) {
 	  printf ("%c[%dm", ESC_E, 40 + bi.bg);
 	  last_bg = bi.bg;
 	}
 	printf (" ");
       } else {
-        /* Figure out the closest character to the biimage */
+	/* Figure out the closest character to the biimage */
 	bool inverse = false;
-        const char *c = block_best (bi, &inverse);
+	const char *c = block_best (bi, &inverse);
 	if (inverse) {
 	  if (last_bg != bi.fg || last_fg != bi.bg) {
 	    printf ("%c[%d;%dm", ESC_E, 30 + bi.bg, 40 + bi.fg);
